// SPDX-License-Identifier: GPL-2.0
/*
 * cdns3-imx.c - imx Glue layer
 *
 * Copyright (C) 2017-2018 NXP
 *
 * Authors: Peter Chen <peter.chen@nxp.com>
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <linux/of.h>
#include <linux/of_platform.h>
#include <linux/io.h>
#include <linux/clk.h>
#include <linux/usb/of.h>
#include <linux/usb/phy.h>
#include <linux/extcon.h>
#include <linux/pm_runtime.h>
#include <linux/reset-controller.h>

#include "cdns3-nxp-reg-def.h"
#include "core.h"

#define USB3_CORE_CTRL1    0x00
#define USB3_CORE_CTRL2    0x04
#define USB3_INT_REG       0x08
#define USB3_CORE_STATUS   0x0c
#define XHCI_DEBUG_LINK_ST 0x10
#define XHCI_DEBUG_BUS     0x14
#define USB3_SSPHY_CTRL1   0x40
#define USB3_SSPHY_CTRL2   0x44
#define USB3_SSPHY_STATUS  0x4c
#define USB2_PHY_CTRL1     0x50
#define USB2_PHY_CTRL2     0x54
#define USB2_PHY_STATUS    0x5c

/* Register bits definition */

/* USB3_CORE_CTRL1 */
#define SW_RESET_MASK	(0x3f << 26)
#define PWR_SW_RESET	(1 << 31)
#define APB_SW_RESET	(1 << 30)
#define AXI_SW_RESET	(1 << 29)
#define RW_SW_RESET	(1 << 28)
#define PHY_SW_RESET	(1 << 27)
#define PHYAHB_SW_RESET	(1 << 26)
#define ALL_SW_RESET	(PWR_SW_RESET | APB_SW_RESET | AXI_SW_RESET | \
		RW_SW_RESET | PHY_SW_RESET | PHYAHB_SW_RESET)
#define OC_DISABLE	(1 << 9)
#define MDCTRL_CLK_SEL	(1 << 7)
#define MODE_STRAP_MASK	(0x7)
#define DEV_MODE	(1 << 2)
#define HOST_MODE	(1 << 1)
#define OTG_MODE	(1 << 0)

/* USB3_INT_REG */
#define CLK_125_REQ	(1 << 29)
#define LPM_CLK_REQ	(1 << 28)
#define DEVU3_WAEKUP_EN	(1 << 14)
#define OTG_WAKEUP_EN	(1 << 12)
#define DEV_INT_EN (3 << 8) /* DEV INT b9:8 */
#define HOST_INT1_EN (1 << 0) /* HOST INT b7:0 */

/* USB3_CORE_STATUS */
#define MDCTRL_CLK_STATUS	(1 << 15)
#define DEV_POWER_ON_READY	(1 << 13)
#define HOST_POWER_ON_READY	(1 << 12)

/* USB3_SSPHY_STATUS */
#define PHY_REFCLK_REQ		(1 << 0)

#define CDNS3_IMX_CLKS_NUM 5
static int cdns3_imx_set_mode(struct cdns3 *cdns);

struct cdns3_imx {
	void __iomem *regs;
	void __iomem *phy_regs;
	enum usb_dr_mode role;
	struct device *dev;
	struct clk *imx_clks[CDNS3_IMX_CLKS_NUM];
	struct reset_controller_dev rcdev;
};

struct cdns3_platform_data cdns3_imx_pdata = {
	.set_mode = cdns3_imx_set_mode,
};

#if 1
static void cdns3_usb_phy_init(void __iomem *regs)
{
	u32 value;

	pr_debug("begin of %s\n", __func__);

	writel(0x0830, regs + PHY_PMA_CMN_CTRL1);
	writel(0x10, regs + TB_ADDR_CMN_DIAG_HSCLK_SEL);
	writel(0x00F0, regs + TB_ADDR_CMN_PLL0_VCOCAL_INIT_TMR);
	writel(0x0018, regs + TB_ADDR_CMN_PLL0_VCOCAL_ITER_TMR);
	writel(0x00D0, regs + TB_ADDR_CMN_PLL0_INTDIV);
	writel(0x4aaa, regs + TB_ADDR_CMN_PLL0_FRACDIV);
	writel(0x0034, regs + TB_ADDR_CMN_PLL0_HIGH_THR);
	writel(0x1ee, regs + TB_ADDR_CMN_PLL0_SS_CTRL1);
	writel(0x7F03, regs + TB_ADDR_CMN_PLL0_SS_CTRL2);
	writel(0x0020, regs + TB_ADDR_CMN_PLL0_DSM_DIAG);
	writel(0x0000, regs + TB_ADDR_CMN_DIAG_PLL0_OVRD);
	writel(0x0000, regs + TB_ADDR_CMN_DIAG_PLL0_FBH_OVRD);
	writel(0x0000, regs + TB_ADDR_CMN_DIAG_PLL0_FBL_OVRD);
	writel(0x0007, regs + TB_ADDR_CMN_DIAG_PLL0_V2I_TUNE);
	writel(0x0027, regs + TB_ADDR_CMN_DIAG_PLL0_CP_TUNE);
	writel(0x0008, regs + TB_ADDR_CMN_DIAG_PLL0_LF_PROG);
	writel(0x0022, regs + TB_ADDR_CMN_DIAG_PLL0_TEST_MODE);
	writel(0x000a, regs + TB_ADDR_CMN_PSM_CLK_CTRL);
	writel(0x139, regs + TB_ADDR_XCVR_DIAG_RX_LANE_CAL_RST_TMR);
	writel(0xbefc, regs + TB_ADDR_XCVR_PSM_RCTRL);

	writel(0x7799, regs + TB_ADDR_TX_PSC_A0);
	writel(0x7798, regs + TB_ADDR_TX_PSC_A1);
	writel(0x509b, regs + TB_ADDR_TX_PSC_A2);
	writel(0x3, regs + TB_ADDR_TX_DIAG_ECTRL_OVRD);
	writel(0x509b, regs + TB_ADDR_TX_PSC_A3);
	writel(0x2090, regs + TB_ADDR_TX_PSC_CAL);
	writel(0x2090, regs + TB_ADDR_TX_PSC_RDY);

	writel(0xA6FD, regs + TB_ADDR_RX_PSC_A0);
	writel(0xA6FD, regs + TB_ADDR_RX_PSC_A1);
	writel(0xA410, regs + TB_ADDR_RX_PSC_A2);
	writel(0x2410, regs + TB_ADDR_RX_PSC_A3);

	writel(0x23FF, regs + TB_ADDR_RX_PSC_CAL);
	writel(0x2010, regs + TB_ADDR_RX_PSC_RDY);

	writel(0x0020, regs + TB_ADDR_TX_TXCC_MGNLS_MULT_000);
	writel(0x00ff, regs + TB_ADDR_TX_DIAG_BGREF_PREDRV_DELAY);
	writel(0x0002, regs + TB_ADDR_RX_SLC_CU_ITER_TMR);
	writel(0x0013, regs + TB_ADDR_RX_SIGDET_HL_FILT_TMR);
	writel(0x0000, regs + TB_ADDR_RX_SAMP_DAC_CTRL);
	writel(0x1004, regs + TB_ADDR_RX_DIAG_SIGDET_TUNE);
	writel(0x4041, regs + TB_ADDR_RX_DIAG_LFPSDET_TUNE2);
	writel(0x0480, regs + TB_ADDR_RX_DIAG_BS_TM);
	writel(0x8006, regs + TB_ADDR_RX_DIAG_DFE_CTRL1);
	writel(0x003f, regs + TB_ADDR_RX_DIAG_ILL_IQE_TRIM4);
	writel(0x543f, regs + TB_ADDR_RX_DIAG_ILL_E_TRIM0);
	writel(0x543f, regs + TB_ADDR_RX_DIAG_ILL_IQ_TRIM0);
	writel(0x0000, regs + TB_ADDR_RX_DIAG_ILL_IQE_TRIM6);
	writel(0x8000, regs + TB_ADDR_RX_DIAG_RXFE_TM3);
	writel(0x0003, regs + TB_ADDR_RX_DIAG_RXFE_TM4);
	writel(0x2408, regs + TB_ADDR_RX_DIAG_LFPSDET_TUNE);
	writel(0x05ca, regs + TB_ADDR_RX_DIAG_DFE_CTRL3);
	writel(0x0258, regs + TB_ADDR_RX_DIAG_SC2C_DELAY);
	writel(0x1fff, regs + TB_ADDR_RX_REE_VGA_GAIN_NODFE);

	writel(0x02c6, regs + TB_ADDR_XCVR_PSM_CAL_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A0BYP_TMR);
	writel(0x02c6, regs + TB_ADDR_XCVR_PSM_A0IN_TMR);
	writel(0x0010, regs + TB_ADDR_XCVR_PSM_A1IN_TMR);
	writel(0x0010, regs + TB_ADDR_XCVR_PSM_A2IN_TMR);
	writel(0x0010, regs + TB_ADDR_XCVR_PSM_A3IN_TMR);
	writel(0x0010, regs + TB_ADDR_XCVR_PSM_A4IN_TMR);
	writel(0x0010, regs + TB_ADDR_XCVR_PSM_A5IN_TMR);

	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A0OUT_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A1OUT_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A2OUT_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A3OUT_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A4OUT_TMR);
	writel(0x0002, regs + TB_ADDR_XCVR_PSM_A5OUT_TMR);

	/* Change rx detect parameter */
	writel(0x960, regs + TB_ADDR_TX_RCVDET_EN_TMR);
	writel(0x01e0, regs + TB_ADDR_TX_RCVDET_ST_TMR);
	writel(0x0090, regs + TB_ADDR_XCVR_DIAG_LANE_FCM_EN_MGN_TMR);

	/* RXDET_IN_P3_32KHZ, Receiver detect slow clock enable */
	value = readl(regs + TB_ADDR_TX_RCVDETSC_CTRL);
	value |= RXDET_IN_P3_32KHZ;
	writel(value, regs + TB_ADDR_TX_RCVDETSC_CTRL);

	udelay(10);

	pr_debug("end of %s\n", __func__);
}
#endif
static void cdns_imx_plat_init(struct cdns3_imx *imx)
{
	u32 value;
	int timeout_us = 100000;

	/* Wait clk value */
	value = readl(imx->regs + USB3_SSPHY_STATUS);
	writel(value, imx->regs + USB3_SSPHY_STATUS);
	udelay(1);
	value = readl(imx->regs + USB3_SSPHY_STATUS);
	while ((value & 0xf0000000) != 0xf0000000 && timeout_us-- > 0) {
		value = readl(imx->regs + USB3_SSPHY_STATUS);
		dev_dbg(imx->dev, "clkvld:0x%x\n", value);
		udelay(1);
	}

	if (timeout_us <= 0)
		dev_err(imx->dev, "wait clkvld timeout\n");

	/* Set all Reset bits */
	value = readl(imx->regs + USB3_CORE_CTRL1);
	value |= ALL_SW_RESET;
	writel(value, imx->regs + USB3_CORE_CTRL1);
	udelay(1);

	value = readl(imx->regs + USB3_CORE_CTRL1);
	value &= ~PHYAHB_SW_RESET;
	writel(value, imx->regs + USB3_CORE_CTRL1);

	cdns3_usb_phy_init(imx->phy_regs);

	value = readl(imx->regs + USB3_CORE_CTRL1);
	value &= ~ALL_SW_RESET;
	writel(value, imx->regs + USB3_CORE_CTRL1);

#if 0
	if (role == USB_DR_MODE_HOST) {
		value = readl(imx->regs + USB3_CORE_CTRL1);
		value = (value & ~MODE_STRAP_MASK) | HOST_MODE | OC_DISABLE;
		writel(value, imx->regs + USB3_CORE_CTRL1);
		value &= ~PHYAHB_SW_RESET;
		writel(value, imx->regs + USB3_CORE_CTRL1);
		mdelay(1);
		/* TODO check the sequence */
		cdns3_usb_phy_init(imx->phy_regs);
		/* Force B Session Valid as 1 */
		writel(0x0060, imx->phy_regs + 0x380a4);
		mdelay(1);

		value = readl(imx->regs + USB3_INT_REG);
		value |= HOST_INT1_EN;
		writel(value, imx->regs + USB3_INT_REG);

		value = readl(imx->regs + USB3_CORE_CTRL1);
		value &= ~ALL_SW_RESET;
		writel(value, imx->regs + USB3_CORE_CTRL1);

		dev_dbg(imx->dev, "wait xhci_power_on_ready\n");

		value = readl(imx->regs + USB3_CORE_STATUS);
		timeout_us = 100000;
		while (!(value & HOST_POWER_ON_READY) && timeout_us-- > 0) {
			value = readl(imx->regs + USB3_CORE_STATUS);
			udelay(1);
		}

		if (timeout_us <= 0)
			dev_err(imx->dev, "wait xhci_power_on_ready timeout\n");

		/*
		value = readl(xhci_regs + XECP_PORT_CAP_REG);
		value |= LPM_2_STB_SWITCH_EN;
		writel(value, xhci_regs + XECP_PORT_CAP_REG);
		*/

		mdelay(1);

		dev_dbg(imx->dev, "switch to host role successfully\n");
	} else if (role == USB_DR_MODE_PERIPHERAL) {
		value = readl(imx->regs + USB3_CORE_CTRL1);
	//	value = (value & ~MODE_STRAP_MASK) | DEV_MODE;
	//	writel(value, imx->regs + USB3_CORE_CTRL1);
		value &= ~PHYAHB_SW_RESET;
		writel(value, imx->regs + USB3_CORE_CTRL1);

		cdns3_usb_phy_init(imx->phy_regs);
		/* Force B Session Valid as 1 */
//		writel(0x0060, imx->phy_regs + 0x380a4);
		value = readl(imx->regs + USB3_INT_REG);
		value |= DEV_INT_EN;
		writel(value, imx->regs + USB3_INT_REG);

		value = readl(imx->regs + USB3_CORE_CTRL1);
		value &= ~ALL_SW_RESET;
		writel(value, imx->regs + USB3_CORE_CTRL1);

		dev_dbg(imx->dev, "wait gadget_power_on_ready\n");

		value = readl(imx->regs + USB3_CORE_STATUS);
		timeout_us = 100000;
		while (!(value & DEV_POWER_ON_READY) && timeout_us-- > 0) {
			value = readl(imx->regs + USB3_CORE_STATUS);
			udelay(1);
		}

		if (timeout_us <= 0)
			dev_err(imx->dev,
				"wait gadget_power_on_ready timeout\n");

		mdelay(1);

		dev_dbg(imx->dev, "switch to gadget role successfully\n");
	}
#endif
}

static int cdns3_imx_set_mode(struct cdns3 *cdns)
{
	struct device *dev = cdns->dev->parent;
	struct cdns3_imx *imx = dev_get_drvdata(dev);
	enum cdns3_roles role = cdns->role;
	u32 value;
	int timeout_us = 100000;

	switch (role) {
	case CDNS3_ROLE_GADGET:
		dev_info(dev, "Set controller to Gadget mode\n");
		value = readl(imx->regs + USB3_CORE_CTRL1);
		value = (value & ~MODE_STRAP_MASK) | DEV_MODE;
		writel(value, imx->regs + USB3_CORE_CTRL1);

		value = readl(imx->regs + USB3_INT_REG);
		value |= DEV_INT_EN;
		writel(value, imx->regs + USB3_INT_REG);

		dev_dbg(dev, "wait gadget_power_on_ready\n");

		value = readl(imx->regs + USB3_CORE_STATUS);
		timeout_us = 100000;
		while (!(value & DEV_POWER_ON_READY) && timeout_us-- > 0) {
			value = readl(imx->regs + USB3_CORE_STATUS);
			udelay(1);
		}

		if (timeout_us <= 0)
			dev_err(dev, "wait gadget_power_on_ready timeout\n");
		break;
	case CDNS3_ROLE_HOST:
		dev_info(dev, "Set controller to Host mode\n");
		value = readl(imx->regs + USB3_CORE_CTRL1);
		value = (value & ~MODE_STRAP_MASK) | HOST_MODE | OC_DISABLE;
		writel(value, imx->regs + USB3_CORE_CTRL1);

		value = readl(imx->regs + USB3_INT_REG);
		value |= HOST_INT1_EN;
		writel(value, imx->regs + USB3_INT_REG);

		dev_dbg(dev, "wait xhci_power_on_ready\n");

		value = readl(imx->regs + USB3_CORE_STATUS);
		timeout_us = 100000;
		while (!(value & HOST_POWER_ON_READY) && timeout_us-- > 0) {
			value = readl(imx->regs + USB3_CORE_STATUS);
			udelay(1);
		}

		if (timeout_us <= 0)
			dev_err(dev, "wait xhci_power_on_ready timeout\n");
		break;
	case CDNS3_ROLE_END:
		dev_info(dev, "reset controller\n");
		cdns_imx_plat_init(imx);
		break;
	}

	return 0;
}

static int imx_get_clks(struct device *dev)
{
	struct cdns3_imx *imx = dev_get_drvdata(dev);
	int ret = 0;

	imx->imx_clks[0] = devm_clk_get(dev, "usb3_lpm_clk");
	if (IS_ERR(imx->imx_clks[0])) {
		ret = PTR_ERR(imx->imx_clks[0]);
		dev_err(dev, "Failed to get usb3_lpm_clk, err=%d\n", ret);
		return ret;
	}

	imx->imx_clks[1] = devm_clk_get(dev, "usb3_bus_clk");
	if (IS_ERR(imx->imx_clks[1])) {
		ret = PTR_ERR(imx->imx_clks[1]);
		dev_err(dev, "Failed to get usb3_bus_clk, err=%d\n", ret);
		return ret;
	}

	imx->imx_clks[2] = devm_clk_get(dev, "usb3_aclk");
	if (IS_ERR(imx->imx_clks[2])) {
		ret = PTR_ERR(imx->imx_clks[2]);
		dev_err(dev, "Failed to get usb3_aclk, err=%d\n", ret);
		return ret;
	}

	imx->imx_clks[3] = devm_clk_get(dev, "usb3_ipg_clk");
	if (IS_ERR(imx->imx_clks[3])) {
		ret = PTR_ERR(imx->imx_clks[3]);
		dev_err(dev, "Failed to get usb3_ipg_clk, err=%d\n", ret);
		return ret;
	}

	imx->imx_clks[4] = devm_clk_get(dev, "usb3_core_pclk");
	if (IS_ERR(imx->imx_clks[4])) {
		ret = PTR_ERR(imx->imx_clks[4]);
		dev_err(dev, "Failed to get usb3_core_pclk, err=%d\n", ret);
		return ret;
	}

	return 0;
}

static int imx_prepare_enable_clks(struct device *dev)
{
	struct cdns3_imx *imx = dev_get_drvdata(dev);
	int i, j, ret = 0;

	for (i = 0; i < CDNS3_IMX_CLKS_NUM; i++) {
		ret = clk_prepare_enable(imx->imx_clks[i]);
		if (ret) {
			dev_err(dev,
				"Failed to prepare/enable cdns3 clk, err=%d\n",
				ret);
			goto err;
		}
	}

	return ret;
err:
	for (j = i; j > 0; j--)
		clk_disable_unprepare(imx->imx_clks[j - 1]);

	return ret;
}

static void imx_disable_unprepare_clks(struct device *dev)
{
	struct cdns3_imx *imx = dev_get_drvdata(dev);
	int i;

	for (i = CDNS3_IMX_CLKS_NUM - 1; i >= 0; i--)
		clk_disable_unprepare(imx->imx_clks[i]);
}

static int cdns3_imx_assert(struct reset_controller_dev *r, unsigned long id)
{
	struct cdns3_imx *imx = container_of(r, struct cdns3_imx, rcdev);
	u32 value;

	value = readl(imx->regs + USB3_CORE_CTRL1);
	if (id == 0)
		value &= ~PHY_SW_RESET;
	else if (id == 1)
		value &= ~PHYAHB_SW_RESET;

	writel(value, imx->regs + USB3_CORE_CTRL1);
	return 0;
}

static int cdns3_imx_deassert(struct reset_controller_dev *r, unsigned long id)
{
	struct cdns3_imx *imx = container_of(r, struct cdns3_imx, rcdev);
	u32 value;

	value = readl(imx->regs + USB3_CORE_CTRL1);
	if (id == 0)
		value &= ~PHY_SW_RESET;
	else if (id == 1)
		value &= ~PHYAHB_SW_RESET;

	writel(value, imx->regs + USB3_CORE_CTRL1);
	return 0;
}

static const struct reset_control_ops cdns3_imx_reset_ops = {
	.assert = cdns3_imx_assert,
	.deassert = cdns3_imx_deassert,
};

/**
 * cdns3_nxp_probe - probe for imx glue driver
 * @pdev: Pointer to imx glue platform device
 *
 * Returns 0 on success otherwise negative errno
 */
static int cdns3_imx_probe(struct platform_device *pdev)
{
	struct device *dev = &pdev->dev;
	struct device_node *node = dev->of_node;
	struct resource	*res;
	struct cdns3_imx *imx;
	void __iomem *regs;
	int ret;
	struct of_dev_auxdata imx_auxdata = {
		.compatible = "cdns,usb3-1.0.0",
		.platform_data = &cdns3_imx_pdata,
	};

	dev_dbg(dev, "%s start, pdata=0x%p\n", __func__, &cdns3_imx_pdata);
	if (!node) {
		dev_err(dev, "device node not found\n");
		return -EINVAL;
	}

	imx = devm_kzalloc(dev, sizeof(*imx), GFP_KERNEL);
	if (!imx)
		return -ENOMEM;

	imx->dev = dev;
	platform_set_drvdata(pdev, imx);

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	regs = devm_ioremap_resource(dev, res);
	if (IS_ERR(regs))
		return PTR_ERR(regs);

	imx->regs = regs;

	res = platform_get_resource(pdev, IORESOURCE_MEM, 1);
	regs = devm_ioremap_resource(dev, res);
	if (IS_ERR(regs))
		return PTR_ERR(regs);

	imx->phy_regs = regs;

	ret = imx_get_clks(dev);
	if (ret)
		return ret;

	ret = imx_prepare_enable_clks(dev);
	if (ret)
		return ret;

//	imx->role = usb_get_dr_mode(dev);
//	dev_dbg(dev, "the init role is %d\n", imx->role);
	cdns_imx_plat_init(imx);

	imx->rcdev.owner = THIS_MODULE;
	imx->rcdev.ops = &cdns3_imx_reset_ops;
	imx->rcdev.of_node = dev->of_node;
	imx->rcdev.nr_resets = 2;
	ret = reset_controller_register(&imx->rcdev);
	if (ret)
		goto err1;

	ret = of_platform_populate(node, NULL, &imx_auxdata, dev);
	if (ret) {
		dev_err(dev, "failed to create cnds3 core\n");
		goto err2;
	}

	dev_dbg(dev, "Cadence USB3 core: probe succeed\n");

	return 0;
err2:
	reset_controller_unregister(&imx->rcdev);
err1:
	imx_disable_unprepare_clks(dev);
	return ret;
}

/**
 * cdns3_imx_remove - remove glue imx driver
 * @pdev: Pointer to Linux platform device
 *
 * Returns 0 on success otherwise negative errno
 */
static int cdns3_imx_remove(struct platform_device *pdev)
{
	imx_disable_unprepare_clks(&pdev->dev);
	return 0;
}

#ifdef CONFIG_OF
static const struct of_device_id of_cdns3_imx_match[] = {
	{ .compatible = "nxp,cdns3" },
	{ },
};
MODULE_DEVICE_TABLE(of, of_cdns3_match);
#endif

static struct platform_driver cdns3_imx_driver = {
	.probe		= cdns3_imx_probe,
	.remove		= cdns3_imx_remove,
	.driver		= {
		.name	= "nxp-cdns3",
		.of_match_table	= of_match_ptr(of_cdns3_imx_match),
	},
};

static int __init cdns3_imx_register(void)
{
	return platform_driver_register(&cdns3_imx_driver);
}
module_init(cdns3_imx_register);

static void __exit cdns3_imx_unregister(void)
{
	platform_driver_unregister(&cdns3_imx_driver);
}
module_exit(cdns3_imx_unregister);

MODULE_ALIAS("platform:cdns3-nxp");
MODULE_AUTHOR("Peter Chen <peter.chen@nxp.com>");
MODULE_LICENSE("GPL v2");
MODULE_DESCRIPTION("Cadence USB3 imx glue driver");
