// SPDX-License-Identifier: GPL-2.0
/*
 * Cadence USBSS DRD Driver.
 *
 * Copyright (C) 2018 Cadence.
 *
 * Author: Pawel Laszczak <pawell@cadence.com>
 *
 */
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/usb/otg.h>
#include <linux/extcon.h>

#include "gadget.h"
#include "core.h"
#include "drd.h"

static int cdns3_get_id(struct cdns3 *cdns)
{
	return !!(readl(&cdns->otg_regs->sts) & OTGSTS_ID_VALUE);
}

static int cdns3_get_vbus(struct cdns3 *cdns)
{
	return !!(readl(&cdns->otg_regs->sts) & OTGSTS_SESSION_VALID);
}

static int cdns3_is_host(struct cdns3 *cdns)
{
	if (cdns->extcon) {
		if (extcon_get_state(cdns->extcon, EXTCON_USB_HOST))
			return 1;
		else
			return 0;
	} else if (!cdns3_get_id(cdns)) {
		return 1;
	} else {
		return 0;
	}
}

static int cdns3_is_device(struct cdns3 *cdns)
{
	if (cdns->extcon) {
		if (extcon_get_state(cdns->extcon, EXTCON_USB))
			return 1;
		else
			return 0;
	} else if (cdns3_get_id(cdns) && cdns3_get_vbus(cdns)) {
		return 1;
	} else {
		return 0;
	}
}

/**
 * cdns3_otg_disable_irq - Disable all OTG interrupts
 * @cdns: Pointer to controller context structure
 */
static void cdns3_otg_disable_irq(struct cdns3 *cdns)
{
	writel(0, &cdns->otg_regs->ien);
}

/**
 * cdns3_otg_enable_irq - enable id and sess_valid interrupts
 * @cdns: Pointer to controller context structure
 */
static void cdns3_enable_id_irq(struct cdns3 *cdns)
{
	writel(OTGIEN_ID_CHANGE_INT, &cdns->otg_regs->ien);
}

static void cdns3_enable_vbus_irq(struct cdns3 *cdns, bool on)
{
	u32 value = readl(&cdns->otg_regs->ien);
	u32 vbus = (OTGIEN_VBUSSESS_RISE_INT | OTGIEN_VBUSSESS_FALL_INT);

	if (on)
		writel(value | vbus, &cdns->otg_regs->ien);
	else
		writel(vbus & ~vbus, &cdns->otg_regs->ien);
}

static void cdns3_drd_switch_off(struct cdns3 *cdns)
{
	/*
	 * driver should wait at least 10us after disabling Device
	 * before turning-off Device (DEV_BUS_DROP)
	 */
	usleep_range(30, 40);
	writel(OTGCMD_HOST_BUS_DROP | OTGCMD_DEV_BUS_DROP |
	       OTGCMD_DEV_POWER_OFF | OTGCMD_HOST_POWER_OFF,
	       &cdns->otg_regs->cmd);
	usleep_range(3000, 4000);
}

static int cdns3_drd_switch_on(struct cdns3 *cdns)
{
	int ret = 0;

	if (cdns->role == CDNS3_ROLE_GADGET) {
		if (cdns->pdata && cdns->pdata->set_mode)
			cdns->pdata->set_mode(cdns);

		writel(OTGCMD_DEV_BUS_REQ | OTGCMD_OTG_DIS,
			&cdns->otg_regs->cmd);

		dev_dbg(cdns->dev, "Waiting for Device mode is turned on\n");

		ret = cdns3_handshake(&cdns->otg_regs->sts, OTGSTS_DEV_READY,
				      OTGSTS_DEV_READY, 100000);
		if (ret)
			return ret;
	} else if (cdns->role == CDNS3_ROLE_HOST) {
		if (cdns->pdata && cdns->pdata->set_mode)
			cdns->pdata->set_mode(cdns);

		writel(OTGCMD_HOST_BUS_REQ | OTGCMD_OTG_DIS, &cdns->otg_regs->cmd);

		ret = cdns3_handshake(&cdns->otg_regs->sts, OTGSTS_XHCI_READY,
				      OTGSTS_XHCI_READY, 100000);
		dev_dbg(cdns->dev, "Waiting for Host mode is turned on, ret=%d\n", ret);
		if (ret)
			return ret;
	} else {
		if (cdns->pdata && cdns->pdata->set_mode)
			cdns->pdata->set_mode(cdns);
	}

	return ret;
}

/**
 * cdns3_init_otg_mode - initialize drd controller
 * @cdns: Pointer to controller context structure
 *
 */
static void cdns3_init_otg_mode(struct cdns3 *cdns)
{
	u32 reg;

	cdns3_otg_disable_irq(cdns);
	/* clear all interrupts */
	writel(~0, &cdns->otg_regs->ivect);

	cdns3_enable_id_irq(cdns);
	if (cdns3_get_id(cdns) == 0)
		cdns3_enable_vbus_irq(cdns, true);

	if (cdns->version == CDNS3_CONTROLLER_V1) {
		reg = readl(&cdns->otg_v1_regs->override);
		reg |= OVERRIDE_IDPULLUP;
		writel(reg, &cdns->otg_v1_regs->override);
	} else {
		reg = readl(&cdns->otg_v0_regs->ctrl1);
		reg |= OVERRIDE_IDPULLUP_V0;
		writel(reg, &cdns->otg_v0_regs->ctrl1);
	}

	/*
	 * Hardware specification says: "ID_VALUE must be valid within
	 * 50ms after idpullup is set to '1" so driver must wait
	 * 50ms before reading this pin.
	 */
	usleep_range(50000, 60000);
}

/**
 * cdns3_drd_irq - interrupt handler for OTG events
 *
 * @irq: irq number for cdns3 core device
 * @data: structure of cdns3
 *
 * Returns IRQ_HANDLED or IRQ_NONE
 */
static irqreturn_t cdns3_drd_irq(int irq, void *data)
{
	irqreturn_t ret = IRQ_NONE;
	struct cdns3 *cdns = data;
	u32 reg;
	bool vbus;

	if (cdns->dr_mode == USB_DR_MODE_HOST)
		return ret;

	reg = readl(&cdns->otg_regs->ivect);

	if (!reg)
		return ret;

	if (reg & OTGIEN_ID_CHANGE_INT) {
		dev_dbg(cdns->dev, "OTG IRQ: new ID: %d\n",
			cdns3_get_id(cdns));

		writel(OTGIEN_ID_CHANGE_INT, &cdns->otg_regs->ivect);
		queue_work(system_freezable_wq, &cdns->role_switch_wq);

		ret = IRQ_HANDLED;
	}

	if ((reg & OTGIEN_VBUSSESS_RISE_INT)
		|| (reg & OTGIEN_VBUSSESS_FALL_INT)) {
		u32 vbus_status;
		vbus = cdns3_get_vbus(cdns);
		dev_dbg(cdns->dev, "OTG IRQ: VBUS : %d\n", vbus);
		vbus_status = vbus ? OTGIEN_VBUSSESS_RISE_INT
			: OTGIEN_VBUSSESS_FALL_INT;
		writel(vbus_status, &cdns->otg_regs->ivect);
		queue_work(system_freezable_wq, &cdns->role_switch_wq);
		ret = IRQ_HANDLED;
	}

	return ret;
}

static int cdns3_extcon_notifier(struct notifier_block *nb, unsigned long event,
			     void *ptr)
{
	struct cdns3 *cdns = container_of(nb, struct cdns3, extcon_nb);

	queue_work(system_freezable_wq, &cdns->role_switch_wq);

	return NOTIFY_DONE;
}

static int cdns3_register_extcon(struct cdns3 *cdns)
{
	struct extcon_dev *extcon;
	struct device *dev = cdns->dev;
	int ret;

	if (of_property_read_bool(dev->of_node, "extcon")) {
		extcon = extcon_get_edev_by_phandle(dev, 0);
		if (IS_ERR(extcon))
			return PTR_ERR(extcon);

		ret = devm_extcon_register_notifier(dev, extcon,
			EXTCON_USB_HOST, &cdns->extcon_nb);
		if (ret < 0) {
			dev_err(dev, "register Host Connector failed\n");
			return ret;
		}

		ret = devm_extcon_register_notifier(dev, extcon,
			EXTCON_USB, &cdns->extcon_nb);
		if (ret < 0) {
			dev_err(dev, "register Device Connector failed\n");
			return ret;
		}

		cdns->extcon = extcon;
		cdns->extcon_nb.notifier_call = cdns3_extcon_notifier;
	}

	return 0;
}

int cdns3_finalize_dr_mode(struct cdns3 *cdns)
{
	void __iomem *regs;

	regs = devm_ioremap_resource(cdns->dev, &cdns->otg_res);
	if (IS_ERR(regs))
		return PTR_ERR(regs);

	/* Detection of DRD version. Controller has been released
	 * in two versions. Both are similar, but they have same changes
	 * in register maps.
	 * The first register in old version is command register and it's read
	 * only, so driver should read 0 from it. On the other hand, in v1
	 * the first register contains device ID number which is not set to 0.
	 * Driver uses this fact to detect the proper version of
	 * controller.
	 */
	cdns->otg_v0_regs = regs;
	if (!readl(&cdns->otg_v0_regs->cmd)) {
		cdns->version  = CDNS3_CONTROLLER_V0;
		cdns->otg_v1_regs = NULL;
		cdns->otg_regs = regs;
		writel(0x1, &cdns->otg_v0_regs->simulate);
		dev_info(cdns->dev, "DRD version v0 (%08x)\n",
			 readl(&cdns->otg_v0_regs->version));
	} else {
		cdns->otg_v0_regs = NULL;
		cdns->otg_v1_regs = regs;
		cdns->otg_regs = (void *)&cdns->otg_v1_regs->cmd;
		cdns->version  = CDNS3_CONTROLLER_V1;
		dev_info(cdns->dev, "DRD version v1 (ID: %08x, rev: %08x)\n",
			 readl(&cdns->otg_v1_regs->did),
			 readl(&cdns->otg_v1_regs->rid));
	}

	cdns->dr_mode = usb_get_dr_mode(cdns->dev);
	if (cdns->dr_mode == USB_DR_MODE_UNKNOWN)
		cdns->dr_mode = USB_DR_MODE_OTG;

	return 0;
}

static int cdns3_do_role_switch(struct cdns3 *cdns, enum cdns3_roles role)
{
	int ret = 0;
	enum cdns3_roles current_role;

	dev_dbg(cdns->dev, "current role is %d, switch to %d\n",
			cdns->role, role);

	if (cdns->role == role)
		return 0;

	current_role = cdns->role;
	cdns3_role_stop(cdns);
	/*
	 * - Enable vbus interrupt after leaving host
	 * - Disable vbus interrupt before entering host
	 */

	if (current_role == CDNS3_ROLE_HOST)
		cdns3_enable_vbus_irq(cdns, true);
	else if (role == CDNS3_ROLE_HOST)
		cdns3_enable_vbus_irq(cdns, false);
	if (role == CDNS3_ROLE_END) {
		/* TODO: Force B Session Valid as 0 */
		return 0;
	}

	ret = cdns3_role_start(cdns, role);
	if (ret) {
		/* Back to current role */
		dev_err(cdns->dev, "set %d has failed, back to %d\n",
					role, current_role);
		if (role == CDNS3_ROLE_HOST)
			cdns3_enable_vbus_irq(cdns, true);
		ret = cdns3_role_start(cdns, current_role);
	}

	return ret;
}

/**
 * cdns3_role_switch - work queue handler for role switch
 *
 * @work: work queue item structure
 *
 * Handles below events:
 * - Role switch for dual-role devices
 * - CDNS3_ROLE_GADGET <--> CDNS3_ROLE_END for peripheral-only devices
 */
static void cdns3_role_switch(struct work_struct *work)
{
	struct cdns3 *cdns = container_of(work, struct cdns3,
			role_switch_wq);
	bool device, host;

	host = cdns3_is_host(cdns);
	device = cdns3_is_device(cdns);

	if (host) {
		if (cdns->roles[CDNS3_ROLE_HOST])
			cdns3_do_role_switch(cdns, CDNS3_ROLE_HOST);
		return;
	}

	if (device)
		cdns3_do_role_switch(cdns, CDNS3_ROLE_GADGET);
	else
		cdns3_do_role_switch(cdns, CDNS3_ROLE_END);
}

int cdns3_drd_init(struct cdns3 *cdns)
{
	int ret;
	u32 state;

	state = readl(&cdns->otg_regs->sts);
	if (OTGSTS_OTG_NRDY(state) != 0) {
		dev_err(cdns->dev, "Cadence USB3 OTG device not ready\n");
		return -ENODEV;
	}

	INIT_WORK(&cdns->role_switch_wq, cdns3_role_switch);
	ret = cdns3_register_extcon(cdns);
	if (ret)
		return ret;

	cdns3_init_otg_mode(cdns);

	return devm_request_threaded_irq(cdns->dev, cdns->irq, cdns3_drd_irq,
					NULL, IRQF_SHARED,
					dev_name(cdns->dev), cdns);
}

int cdns3_role_start(struct cdns3 *cdns, enum cdns3_roles role)
{
	int ret;

	if (role >= CDNS3_ROLE_END) {
		WARN_ON(role > CDNS3_ROLE_END);
		return 0;
	}

	if (!cdns->roles[role])
		return -ENXIO;

	if (cdns->roles[role]->state == CDNS3_ROLE_STATE_ACTIVE)
		return 0;

	mutex_lock(&cdns->mutex);
	cdns->role = role;
	cdns3_drd_switch_on(cdns);
	ret = cdns->roles[role]->start(cdns);
	if (!ret)
		cdns->roles[role]->state = CDNS3_ROLE_STATE_ACTIVE;
	mutex_unlock(&cdns->mutex);
	return ret;
}

void cdns3_role_stop(struct cdns3 *cdns)
{
	enum cdns3_roles role = cdns->role;

	if (role >= CDNS3_ROLE_END) {
		WARN_ON(role > CDNS3_ROLE_END);
		return;
	}

	if (cdns->roles[role]->state == CDNS3_ROLE_STATE_INACTIVE)
		return;

	mutex_lock(&cdns->mutex);
	cdns->roles[role]->stop(cdns);
	cdns->roles[role]->state = CDNS3_ROLE_STATE_INACTIVE;
	cdns3_drd_switch_off(cdns);
	mutex_unlock(&cdns->mutex);
}

/*
 * cdns->role gets from cdns3_get_role, and this API tells role at the
 * runtime.
 * If both roles are supported, the role is selected based on vbus/id.
 * It could be read from OTG register or external connector.
 * If only single role is supported, only one role structure
 * is allocated, cdns->roles[CDNS3_ROLE_HOST] or cdns->roles[CDNS3_ROLE_GADGET].
 */
enum cdns3_roles cdns3_get_role(struct cdns3 *cdns)
{
	if (cdns->roles[CDNS3_ROLE_HOST] && cdns->roles[CDNS3_ROLE_GADGET]) {
		if (cdns3_is_host(cdns))
			return CDNS3_ROLE_HOST;
		else if (cdns3_is_device(cdns))
			return CDNS3_ROLE_GADGET;
		else
			return CDNS3_ROLE_END;
	} else {
		return cdns->roles[CDNS3_ROLE_HOST]
			? CDNS3_ROLE_HOST
			: CDNS3_ROLE_GADGET;
	}
}
